# -*- coding: utf-8 -*-

# DO NOT EDIT THIS FILE!
# This file has been autogenerated by dephell <3
# https://github.com/dephell/dephell

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

import os.path

readme = ''
here = os.path.abspath(os.path.dirname(__file__))
readme_path = os.path.join(here, 'README.rst')
if os.path.exists(readme_path):
    with open(readme_path, 'rb') as stream:
        readme = stream.read().decode('utf8')

setup(
    long_description=readme,
    name='miblepy',
    version='0.4.6',
    description='miblepy fetches data from various (Xiaomi/Mijia/Mi) Bluetooth LE devices and push it to a MQTT broker in a coordinated, sequential manner.',
    python_requires='==3.*,>=3.8.0',
    project_urls={
        "homepage": "https://github.com/benleb/miblepy",
        "repository": "https://github.com/benleb/miblepy"
    },
    author='Ben Lebherz',
    author_email='git@benleb.de',
    license='MIT',
    keywords='bluetooth smart-home sensor xiaomi home-assistant',
    classifiers=[
        'Development Status :: 4 - Beta', 'Environment :: Console',
        'Intended Audience :: Developers', 'Intended Audience :: Education',
        'License :: OSI Approved :: MIT License',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8', 'Topic :: Home Automation'
    ],
    entry_points={"console_scripts": ["mible = miblepy.mible.cli:cli"]},
    packages=[
        'miblepy', 'miblepy.devices', 'miblepy.devices.xbm', 'miblepy.mible'
    ],
    package_dir={"": "."},
    package_data={},
    install_requires=[
        'bluepy==1.*,>=1.3.0', 'click==7.*,>=7.1.2', 'paho-mqtt==1.*,>=1.5.0',
        'tomlkit==0.*,>=0.6.0'
    ],
    extras_require={
        "dev": [
            "bandit==1.*,>=1.6.2", "black==19.*,>=19.10.0.b0",
            "flake8==3.*,>=3.8.1", "isort==4.*,>=4.3.21", "mypy==0.*,>=0.770.0",
            "pycodestyle==2.*,>=2.6.0", "pytest==5.*,>=5.2.0"
        ]
    },
)
